@@ -361,6 +361,8 @@ static int dhcp_client_send_discover(struct l_dhcp_client *client)
 						client->hostname))
 			return -EINVAL;
 
+	_dhcp_message_builder_append(&builder, DHCP_OPTION_RAPID_COMMIT,
+					0, "");
 	_dhcp_message_builder_finalize(&builder, &len);
 
 	return client->transport->l2_send(client->transport,
@@ -671,6 +673,7 @@ static void dhcp_client_address_add_cb(int error, uint16_t type,
 }
 
 static int dhcp_client_receive_ack(struct l_dhcp_client *client,
+					const uint8_t *saddr,
 					const struct dhcp_message *ack,
 					size_t len)
 {
@@ -695,6 +698,9 @@ static int dhcp_client_receive_ack(struct l_dhcp_client *client,
 
 	lease->address = ack->yiaddr;
 
+	if (saddr)
+		memcpy(lease->server_mac, saddr, ETH_ALEN);
+
 	r = L_DHCP_CLIENT_EVENT_LEASE_RENEWED;
 
 	if (client->lease) {
@@ -719,15 +725,11 @@ static int dhcp_client_receive_ack(struct l_dhcp_client *client,
 			l_dhcp_lease_get_address(client->lease);
 		uint8_t prefix_len;
 		uint32_t l = l_dhcp_lease_get_lifetime(client->lease);
-		L_AUTO_FREE_VAR(char *, netmask) =
-				l_dhcp_lease_get_netmask(client->lease);
 		L_AUTO_FREE_VAR(char *, broadcast) =
 				l_dhcp_lease_get_broadcast(client->lease);
-		struct in_addr in_addr;
 
-		if (inet_pton(AF_INET, netmask, &in_addr) > 0)
-			prefix_len = __builtin_popcountl(in_addr.s_addr);
-		else
+		prefix_len = l_dhcp_lease_get_prefix_length(client->lease);
+		if (!prefix_len)
 			prefix_len = 24;
 
 		a = l_rtnl_address_new(ip, prefix_len);
@@ -773,7 +775,8 @@ static int dhcp_client_receive_offer(struct l_dhcp_client *client,
 	return 0;
 }
 
-static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
+static void dhcp_client_rx_message(const void *data, size_t len, void *userdata,
+					const uint8_t *saddr)
 {
 	struct l_dhcp_client *client = userdata;
 	const struct dhcp_message *message = data;
@@ -815,6 +818,17 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 	case DHCP_STATE_INIT:
 		return;
 	case DHCP_STATE_SELECTING:
+		if (msg_type == DHCP_MESSAGE_TYPE_ACK) {
+			_dhcp_message_iter_init(&iter, message, len);
+
+			while (_dhcp_message_iter_next(&iter, &t, &l, &v))
+				if (t == DHCP_OPTION_RAPID_COMMIT) {
+					CLIENT_ENTER_STATE(
+							DHCP_STATE_REQUESTING);
+					goto receive_rapid_commit;
+				}
+		}
+
 		if (msg_type != DHCP_MESSAGE_TYPE_OFFER)
 			return;
 
@@ -835,6 +849,7 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 	case DHCP_STATE_REQUESTING:
 	case DHCP_STATE_RENEWING:
 	case DHCP_STATE_REBINDING:
+	receive_rapid_commit:
 		if (msg_type == DHCP_MESSAGE_TYPE_NAK) {
 			CLIENT_DEBUG("Received NAK, Stopping...");
 			l_dhcp_client_stop(client);
@@ -847,7 +862,7 @@ static void dhcp_client_rx_message(const void *data, size_t len, void *userdata)
 		if (msg_type != DHCP_MESSAGE_TYPE_ACK)
 			return;
 
-		r = dhcp_client_receive_ack(client, message, len);
+		r = dhcp_client_receive_ack(client, saddr, message, len);
 		if (r < 0)
 			return;
 
